/*
 * Copyright (C) 2006-2007 Zimbra, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU Lesser General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string.h>

#include "e-cal-backend-zimbra-factory.h"
#include "e-cal-backend-zimbra.h"
#include <libezimbra/e-zimbra-log.h>

typedef struct _ECalBackendZimbraFactory ECalBackendZimbraFactory;
typedef struct _ECalBackendZimbraFactoryClass ECalBackendZimbraFactoryClass;

struct _ECalBackendZimbraFactory
{
        ECalBackendFactory parent;
};

struct _ECalBackendZimbraFactoryClass
{
        ECalBackendFactoryClass parent_class;
};

static GType zimbra_types[2];

static const gchar *
factory_get_protocol (ECalBackendFactory *factory)
{
        return "zimbra";
}

static ECalBackend *
factory_todos_new_backend (ECalBackendFactory *factory,
                           ESource *source)
{
        return g_object_new (
                E_TYPE_CAL_BACKEND_ZIMBRA, "source", source,
                "kind", ICAL_VTODO_COMPONENT, NULL);
}

static icalcomponent_kind
factory_todos_get_kind (ECalBackendFactory *factory)
{
        return ICAL_VTODO_COMPONENT;
}

static ECalBackend*
factory_events_new_backend (ECalBackendFactory *factory,
                            ESource *source)
{
        return g_object_new (
                E_TYPE_CAL_BACKEND_ZIMBRA, "source", source,
                "kind", ICAL_VEVENT_COMPONENT, NULL);
}

static icalcomponent_kind
factory_events_get_kind (ECalBackendFactory *factory)
{
        return ICAL_VEVENT_COMPONENT;
}

static void
todos_backend_factory_class_init (ECalBackendZimbraFactoryClass *class)
{
        ECalBackendFactoryClass *backend_factory_class;

        backend_factory_class = E_CAL_BACKEND_FACTORY_CLASS (class);
        backend_factory_class->get_protocol = factory_get_protocol;
        backend_factory_class->get_kind = factory_todos_get_kind;
        backend_factory_class->new_backend = factory_todos_new_backend;
}

static void
events_backend_factory_class_init (ECalBackendZimbraFactoryClass *class)
{
        ECalBackendFactoryClass *backend_factory_class;

        backend_factory_class = E_CAL_BACKEND_FACTORY_CLASS (class);
        backend_factory_class->get_protocol = factory_get_protocol;
        backend_factory_class->get_kind = factory_events_get_kind;
        backend_factory_class->new_backend = factory_events_new_backend;
}

static GType
events_backend_factory_get_type (GTypeModule *module)
{
        GType type;

        GTypeInfo type_info =
        {
                sizeof (ECalBackendZimbraFactoryClass),
                (GBaseInitFunc) NULL,
                (GBaseFinalizeFunc) NULL,
                (GClassInitFunc) events_backend_factory_class_init,
                (GClassFinalizeFunc) NULL,
                NULL, /* class_data */
                sizeof (ECalBackend),
                0,    /* n_preallocs */
                (GInstanceInitFunc) NULL,
                NULL  /* value_table */
        };

        type = g_type_module_register_type (
                module, E_TYPE_CAL_BACKEND_FACTORY,
                "ECalBackendZimbraEventsFactory", &type_info, 0);

        return type;
}

static GType
todos_backend_factory_get_type (GTypeModule *module)
{
        GType type;

        GTypeInfo type_info =
        {
                sizeof (ECalBackendZimbraFactoryClass),
                (GBaseInitFunc) NULL,
                (GBaseFinalizeFunc) NULL,
                (GClassInitFunc) todos_backend_factory_class_init,
                (GClassFinalizeFunc) NULL,
                NULL,  /* class_data */
                sizeof (ECalBackend),
                0,     /* n_preallocs */
                (GInstanceInitFunc) NULL,
                NULL   /* value_table */
        };

        type = g_type_module_register_type (
                module, E_TYPE_CAL_BACKEND_FACTORY,
                "ECalBackendZimbraTodosFactory", &type_info, 0);

        return type;
}

void
eds_module_initialize (GTypeModule *module)
{
        zimbra_types[0] = events_backend_factory_get_type (module);

        glog_init();
}

void
eds_module_shutdown (void)
{
}

void
eds_module_list_types (const GType **types,
                       gint *num_types)
{
        *types = zimbra_types;
        *num_types = 1;
}
