/* 
 * Copyright (C) 2006-2007 Zimbra, Inc.
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of version 2 of the GNU Lesser General Public 
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 * 
 */

#ifndef E_ZIMBRA_UTILS_H
#define E_ZIMBRA_UTILS_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <glib.h>
#include <libedataserver/e-url.h>
#include <glib/gstdio.h>
#include <time.h>

G_BEGIN_DECLS

typedef enum
{
        E_FILE_CACHE_UPDATE_IDS,
        E_FILE_CACHE_DELETE_IDS
} EFileCacheIDType;

#ifdef HAVE_LIBEBACKEND
#include <libebackend/e-file-cache.h>
#else
#include <libedataserver/e-file-cache.h>
#endif

gboolean        e_zimbra_utils_find_cache_string(EFileCache *cache,
                                                 const gchar *key,
                                                 const gchar *str);

void            e_zimbra_utils_del_cache_string (EFileCache *cache,
                                                 const gchar *key,
                                                 const gchar *str);

GPtrArray *     e_zimbra_utils_get_cache_array  (EFileCache *cache,
                                                 const gchar *key);

GPtrArray *     e_zimbra_utils_make_array_from_string
                                                (const gchar *string);

const gchar *   e_zimbra_utils_check_array_for_string
                                                (GPtrArray *array,
                                                 const gchar *string);

gchar *         e_zimbra_utils_make_string_from_array
                                                (GPtrArray * array);

gchar *         e_zimbra_utils_uri_to_fspath    (const gchar *uri);

void            e_zimbra_utils_pack_id          (gchar *packed_id,
                                                 gsize packed_id_len,
                                                 const gchar *zid,
                                                 const gchar *rev,
                                                 time_t md);

void            e_zimbra_utils_unpack_id        (const gchar *packed_id,
                                                 const gchar **zid,
                                                 const gchar **rev,
                                                 time_t *md);

GPtrArray *     e_file_cache_get_ids            (EFileCache *cache,
                                                 EFileCacheIDType type);

gboolean        e_file_cache_set_ids            (EFileCache *cache,
                                                 EFileCacheIDType type,
                                                 GPtrArray *ids);

gboolean        e_file_cache_add_ids            (EFileCache *cache,
                                                 EFileCacheIDType type,
                                                 const gchar *ids);

const gchar *   g_ptr_array_lookup_id           (GPtrArray *array,
                                                 const gchar *id);

gboolean        g_ptr_array_remove_id           (GPtrArray *array,
                                                 const gchar *id);

gchar *         path_from_uri                   (const gchar *uri);

void            e_uri_set_path                  (EUri *uri,
                                                 const gchar *path);

void            g_string_append_url_encoded     (GString * str,
                                                 const gchar *in,
                                                 const gchar *extra_enc_chars);

gboolean        g_string_unescape               (GString *string,
                                                 const gchar *illegal_characters);

gboolean        zimbra_parse_version_string     (const gchar *version,
                                                 guint *major,
                                                 guint *minor,
                                                 guint *micro);

gboolean        zimbra_check_min_server_version (gchar *version_string);

void            zimbra_recursive_delete         (const gchar *path);


gchar *         e_zimbra_encode_url             (const gchar *url,
                                                 gchar *encoded,
                                                 gsize encoded_len);

G_END_DECLS

#endif /* E_ZIMBRA_UTILS_H */
